@@ -48,6 +48,9 @@ def _get_options(*args, **kwargs):
     else:
         dsn = None
 
+    if len(args) > 1:
+        raise TypeError("Only single positional argument is expected")
+
     rv = dict(DEFAULT_OPTIONS)
     options = dict(*args, **kwargs)
     if dsn is not None and options.get("dsn") is None:
@@ -260,6 +263,13 @@ class _Client(object):
         if ignored_by_config_option:
             return False
 
+        return True
+
+    def _should_sample_error(
+        self,
+        event,  # type: Event
+    ):
+        # type: (...) -> bool
         not_in_sample_rate = (
             self.options["sample_rate"] < 1.0
             and random.random() >= self.options["sample_rate"]
@@ -349,9 +359,13 @@ class _Client(object):
         if session:
             self._update_session_from_event(session, event)
 
-        attachments = hint.get("attachments")
         is_transaction = event_opt.get("type") == "transaction"
 
+        if not is_transaction and not self._should_sample_error(event):
+            return None
+
+        attachments = hint.get("attachments")
+
         # this is outside of the `if` immediately below because even if we don't
         # use the value, we want to make sure we remove it before the event is
         # sent
@@ -359,6 +373,12 @@ class _Client(object):
             event_opt.get("contexts", {}).get("trace", {}).pop("tracestate", "")
         )
 
+        dynamic_sampling_context = (
+            event_opt.get("contexts", {})
+            .get("trace", {})
+            .pop("dynamic_sampling_context", {})
+        )
+
         # Transactions or events with attachments should go to the /envelope/
         # endpoint.
         if is_transaction or attachments:
@@ -368,15 +388,23 @@ class _Client(object):
                 "sent_at": format_timestamp(datetime.utcnow()),
             }
 
-            tracestate_data = raw_tracestate and reinflate_tracestate(
-                raw_tracestate.replace("sentry=", "")
-            )
-            if tracestate_data and has_tracestate_enabled():
-                headers["trace"] = tracestate_data
+            if has_tracestate_enabled():
+                tracestate_data = raw_tracestate and reinflate_tracestate(
+                    raw_tracestate.replace("sentry=", "")
+                )
+
+                if tracestate_data:
+                    headers["trace"] = tracestate_data
+            elif dynamic_sampling_context:
+                headers["trace"] = dynamic_sampling_context
 
             envelope = Envelope(headers=headers)
 
             if is_transaction:
+                if "profile" in event_opt:
+                    event_opt["profile"]["transaction_id"] = event_opt["event_id"]
+                    event_opt["profile"]["version_name"] = event_opt["release"]
+                    envelope.add_profile(event_opt.pop("profile"))
                 envelope.add_transaction(event_opt)
             else:
                 envelope.add_event(event_opt)
