@@ -11,6 +11,7 @@ from sentry_sdk._compat import PY2, reraise, iteritems
 from sentry_sdk.tracing import Transaction
 from sentry_sdk.sessions import auto_session_tracking
 from sentry_sdk.integrations._wsgi_common import _filter_headers
+from sentry_sdk.profiler import profiling
 
 from sentry_sdk._types import MYPY
 
@@ -127,7 +128,7 @@ class SentryWsgiMiddleware(object):
 
                     with hub.start_transaction(
                         transaction, custom_sampling_context={"wsgi_environ": environ}
-                    ):
+                    ), profiling(transaction, hub):
                         try:
                             rv = self.app(
                                 environ,
